@@ -21,9 +21,8 @@ from psycopg2.sql import SQL, Identifier, Composable
 from tqdm import tqdm
 
 from splitgraph.config import SPLITGRAPH_META_SCHEMA, SPLITGRAPH_API_SCHEMA, SG_CMD_ASCII
-from splitgraph.core.common import Tracer
+from splitgraph.core.common import Tracer, get_temporary_table_id
 from splitgraph.core.fragment_manager import (
-    get_temporary_table_id,
     get_chunk_groups,
     ExtraIndexInfo,
 )
@@ -32,6 +31,7 @@ from splitgraph.core.output import pluralise, truncate_list
 from splitgraph.core.sql import select
 from splitgraph.core.types import TableSchema, Quals
 from splitgraph.engine import ResultShape
+from splitgraph.engine.postgres.engine import get_change_key
 from splitgraph.exceptions import ObjectIndexingError
 
 if TYPE_CHECKING:
@@ -39,7 +39,6 @@ if TYPE_CHECKING:
     from splitgraph.core.repository import Repository
     from splitgraph.engine.postgres.engine import PostgresEngine
 
-
 # Output checkout progress every 5MB to trade off between the latency
 # of initializing a batch of object applications and not reporting anything at all
 _PROGRESS_EVERY = 5 * 1024 * 1024
@@ -101,35 +100,6 @@ def _empty_callback(**kwargs) -> None:
     pass
 
 
-def create_foreign_table(
-    schema: str,
-    server: str,
-    table_name: str,
-    schema_spec: TableSchema,
-    internal_table_name: Optional[str] = None,
-    extra_options: Optional[Dict[str, str]] = None,
-):
-    table_options = extra_options or {}
-    table_options.update({"table": internal_table_name or table_name})
-
-    table_opts, table_optvals = zip(*table_options.items())
-
-    query = SQL("CREATE FOREIGN TABLE {}.{} (").format(Identifier(schema), Identifier(table_name))
-    query += SQL(",".join("{} %s " % col.pg_type for col in schema_spec)).format(
-        *(Identifier(col.name) for col in schema_spec)
-    )
-    query += SQL(") SERVER {} OPTIONS (").format(Identifier(server))
-    query += SQL(",").join(Identifier(o) + SQL(" %s") for o in table_opts) + SQL(");")
-    args = list(table_optvals)
-    for col in schema_spec:
-        if col.comment:
-            query += SQL("COMMENT ON COLUMN {}.{}.{} IS %s;").format(
-                Identifier(schema), Identifier(table_name), Identifier(col.name),
-            )
-            args.append(col.comment)
-    return query, args
-
-
 class QueryPlan:
     """
     Represents the initial query plan (fragments to query) for given columns and
@@ -194,9 +164,7 @@ class QueryPlan:
 
     def _extract_singleton_fragments(self) -> Tuple[List[str], List[str]]:
         # Get fragment boundaries (min-max PKs of every fragment).
-        table_pk = [(t[1], t[2]) for t in self.table.table_schema if t[3]]
-        if not table_pk:
-            table_pk = [(t[1], t[2]) for t in self.table.table_schema]
+        table_pk = get_change_key(self.table.table_schema)
         object_pks = self.object_manager.get_min_max_pks(self.filtered_objects, table_pk)
         # Group fragments into non-overlapping groups: those can be applied independently of each other.
         object_groups = get_chunk_groups(
@@ -302,6 +270,7 @@ class Table:
         destination: str,
         destination_schema: Optional[str] = None,
         lq_server: Optional[str] = None,
+        temporary: bool = False,
     ) -> None:
         """
         Materializes a Splitgraph table in the target schema as a normal Postgres table, potentially downloading all
@@ -311,6 +280,9 @@ class Table:
         :param destination_schema: Name of the destination schema.
         :param lq_server: If set, sets up a layered querying FDW for the table instead using this foreign server.
         """
+        # Circular import
+        from splitgraph.hooks.data_source.fdw import create_foreign_table
+
         destination_schema = destination_schema or self.repository.to_schema()
         engine = self.repository.object_engine
         object_manager = self.repository.objects
@@ -327,6 +299,7 @@ class Table:
                     schema_spec=self.table_schema,
                     include_comments=True,
                     unlogged=True,
+                    temporary=temporary,
                 )
                 if required_objects:
                     logging.debug("Applying %s...", pluralise("fragment", len(required_objects)))
@@ -349,7 +322,11 @@ class Table:
                     )
         else:
             query, args = create_foreign_table(
-                destination_schema, lq_server, self.table_name, self.table_schema
+                destination_schema,
+                lq_server,
+                self.table_name,
+                self.table_schema,
+                extra_options={"table": self.table_name},
             )
 
             engine.run_sql(query, args)
